@@ -1,6 +1,7 @@
 /*
  * This file is part of libaacs
  * Copyright (C) 2009-2010  Obliter0n
+ * Copyright (C) 2009-2013  npzacz
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -35,6 +36,7 @@
 #define AACS_ERROR_CERT_REVOKED   -5 /* certificate has been revoked */
 #define AACS_ERROR_MMC_OPEN       -6 /* MMC open failed (no MMC drive ?) */
 #define AACS_ERROR_MMC_FAILURE    -7 /* MMC failed */
+#define AACS_ERROR_NO_DK          -8 /* no matching device key */
 
 typedef struct aacs AACS;
 
@@ -51,6 +53,11 @@ AACS_PUBLIC int aacs_get_mkb_version(AACS *aacs);
 AACS_PUBLIC const uint8_t *aacs_get_disc_id(AACS *aacs);
 AACS_PUBLIC const uint8_t *aacs_get_vid(AACS *aacs);  /* may fail even if disc can be decrypted */
 AACS_PUBLIC const uint8_t *aacs_get_pmsn(AACS *aacs); /* may fail even if disc can be decrypted */
+AACS_PUBLIC const uint8_t *aacs_get_mk(AACS *aacs);   /* may fail even if disc can be decrypted */
+
+/* AACS Online */
+AACS_PUBLIC const uint8_t *aacs_get_device_binding_id(AACS *aacs);
+AACS_PUBLIC const uint8_t *aacs_get_device_nonce(AACS *aacs);
 
 /* revocation lists */
 typedef struct {
@@ -60,5 +67,13 @@ typedef struct {
 
 AACS_PUBLIC AACS_RL_ENTRY *aacs_get_hrl(int *num_entries, int *mkb_version);
 AACS_PUBLIC AACS_RL_ENTRY *aacs_get_drl(int *num_entries, int *mkb_version);
+AACS_PUBLIC void           aacs_free_rl(AACS_RL_ENTRY **rl);
+
+/* bus encryption info */
+
+#define AACS_BUS_ENCRYPTION_ENABLED  0x01  /* Bus encryption enabled in the media */
+#define AACS_BUS_ENCRYPTION_CAPABLE  0x02  /* Bus encryption capable drive */
+
+AACS_PUBLIC uint32_t aacs_get_bus_encryption(AACS *);
 
 #endif /* AACS_H_ */
