@@ -2,7 +2,7 @@
 /*
  * This file is part of libaacs
  * Copyright (C) 2009-2010  Obliter0n
- * Copyright (C) 2010       npzacs
+ * Copyright (C) 2010-2013  npzacs
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,7 @@
 
 #include "mmc.h"
 #include "crypto.h"
+#include "file/path.h"
 #include "util/macro.h"
 #include "util/logging.h"
 
@@ -89,11 +90,6 @@
 #include <winsock.h>
 #endif
 
-/* define in CFLAGS to skip drive certificate checks */
-#ifndef PATCHED_DRIVE
-#define PATCHED_DRIVE 0
-#endif
-
 #ifndef DEBUG_KEYS
 #define DEBUG_KEYS 0
 #endif
@@ -146,6 +142,10 @@ struct mmc {
     uint8_t host_nonce[20];
     uint8_t host_key[20];
     uint8_t host_key_point[40];
+
+    uint8_t drive_cert[92];
+
+    uint8_t read_drive_cert;
 };
 
 static int _mmc_send_cmd(MMC *mmc, const uint8_t *cmd, uint8_t *buf, size_t tx,
@@ -388,6 +388,8 @@ static int _mmc_report_disc_structure(MMC *mmc, uint8_t agid, uint8_t format,
     memset(cmd, 0, sizeof(cmd));
     memset(buf, 0, len);
 
+    DEBUG(DBG_MMC, "MMC report disc structure [format 0x%x layer %d address %d] ...\n", format, layer, address);
+
     cmd[0] = 0xad; // operation code
     cmd[1] = 0x01; // BluRay
     cmd[2] = (address >> 24) & 0xff;
@@ -409,6 +411,8 @@ static int _mmc_get_configuration(MMC *mmc, uint16_t feature, uint8_t *buf, uint
     memset(cmd, 0, sizeof(cmd));
     memset(buf, 0, len);
 
+    DEBUG(DBG_MMC, "MMC get configuration [feature 0x%x] ...\n", feature);
+
     cmd[0] = 0x46; // operation code
     cmd[1] = 0x01; // BluRay
     cmd[2] = (feature >> 8) & 0xff;
@@ -467,8 +471,8 @@ static int _mmc_send_host_cert(MMC *mmc, uint8_t agid,
     return _mmc_send_key(mmc, agid, 0x01, buf, 116);
 }
 
-static int _mmc_read_drive_cert(MMC *mmc, uint8_t agid, uint8_t *drive_nonce,
-                                uint8_t *drive_cert)
+static int _mmc_read_drive_cert_challenge(MMC *mmc, uint8_t agid, uint8_t *drive_nonce,
+                                          uint8_t *drive_cert)
 {
     uint8_t buf[116];
     memset(buf, 0, sizeof(buf));
@@ -505,13 +509,16 @@ static int _mmc_check_aacs(MMC *mmc)
         uint16_t feature = MKINT_BE16(buf+8);
         if (feature == 0x010d) {
             DEBUG(DBG_MMC, "AACS feature descriptor:\n");
-            DEBUG(DBG_MMC, "  AACS version: %d\n", buf[5+8]);
+            DEBUG(DBG_MMC, "  AACS version: %d\n", buf[7+8]);
             DEBUG(DBG_MMC, "  AACS active: %d\n", buf[2+8] & 1);
             DEBUG(DBG_MMC, "  Binding Nonce generation support: %d\n", buf[4+8] & 1);
             DEBUG(DBG_MMC, "  Binding Nonce block count: %d\n", buf[5+8]);
-            DEBUG(DBG_MMC, "  Bus encryption support: %d\n", buf[4+8] & 2);
+            DEBUG(DBG_MMC, "  Bus encryption support: %d\n", !!(buf[4+8] & 2));
+            DEBUG(DBG_MMC, "  Read drive certificate: %d\n", !!(buf[4+8] & 0x10));
             DEBUG(DBG_MMC, "  AGID count: %d\n", buf[6+8] & 0xf);
 
+            mmc->read_drive_cert = !!(buf[4+8] & 0x10);
+
             return buf[2+8] & 1;
         }
         DEBUG(DBG_MMC, "incorrect feature ID %04x\n", feature);
@@ -597,6 +604,19 @@ static int _mmc_read_pmsn(MMC *mmc, uint8_t agid, uint8_t *pmsn,
     return 0;
 }
 
+static int _mmc_read_data_keys(MMC *mmc, uint8_t agid, uint8_t *read_data_key, uint8_t *write_data_key)
+{
+    uint8_t buf[36];
+
+    if (_mmc_report_disc_structure(mmc, agid, 0x84, 0, 0, buf, 36)) {
+        memcpy(read_data_key, buf + 4, 16);
+        memcpy(write_data_key, buf + 20, 16);
+        return 1;
+    }
+
+    return 0;
+}
+
 #ifdef USE_IOKIT
 static int get_mounted_device_from_path (MMC *mmc, const char *path) {
   struct statfs stat_info;
@@ -857,18 +877,12 @@ MMC *mmc_open(const char *path)
 
 #elif defined(HAVE_MNTENT_H)
 
-#ifdef HAVE_REALPATH
-    char *file_path = malloc(PATH_MAX);
-    if (!file_path || !realpath(path, file_path)) {
+    char file_path [AACS_PATH_MAX];
+    if (!aacs_resolve_path(path, file_path)) {
         DEBUG(DBG_MMC | DBG_CRIT, "Failed resolving path %s\n", path);
         X_FREE(mmc);
-        X_FREE(file_path);
         return NULL;
     }
-#else
-    char *file_path = (char*)malloc(strlen(path) + 1);
-    strcpy(file_path, path);
-#endif
 
     int   path_len  = strlen(file_path);
     FILE *proc_mounts;
@@ -909,8 +923,6 @@ MMC *mmc_open(const char *path)
         X_FREE(mmc);
     }
 
-    X_FREE(file_path);
-
 #elif defined(_WIN32)
     char drive[] = { path[0], ':', '\\', 0 };
     char volume[] = {'\\', '\\', '.', '\\', path[0], ':', 0};
@@ -948,13 +960,17 @@ MMC *mmc_open(const char *path)
 
 #endif
 
-#ifndef _WIN32
     if (mmc && !_mmc_check_aacs(mmc)) {
         DEBUG(DBG_MMC | DBG_CRIT, "AACS not active or supported by the drive\n");
+#ifndef _WIN32
         mmc_close (mmc);
         return NULL;
-    }
 #endif
+    }
+
+    if (mmc && mmc->read_drive_cert) {
+        mmc_read_drive_cert(mmc, mmc->drive_cert);
+    }
 
     return mmc;
 }
@@ -989,10 +1005,17 @@ void mmc_close(MMC *mmc)
 
         DEBUG(DBG_MMC, "Closed MMC drive\n");
 
+        /* erase sensitive data */
+        memset(mmc, 0, sizeof(*mmc));
+
         X_FREE(mmc);
     }
 }
 
+/*
+ *
+ */
+
 static int _verify_signature(const uint8_t *cert, const uint8_t *signature,
                              const uint8_t *nonce, const uint8_t *point)
 {
@@ -1004,16 +1027,187 @@ static int _verify_signature(const uint8_t *cert, const uint8_t *signature,
     return crypto_aacs_verify(cert, signature, data, 60);
 }
 
-int mmc_read_vid(MMC *mmc, const uint8_t *host_priv_key, const uint8_t *host_cert, uint8_t *vid, uint8_t *pmsn)
+static int _mmc_aacs_auth(MMC *mmc, uint8_t agid, const uint8_t *host_priv_key, const uint8_t *host_cert, uint8_t *bus_key)
 {
-    uint8_t agid = 0, hks[40], dn[20], dc[92], dkp[40], dks[40], mac[16];
+    uint8_t hks[40], dn[20], dkp[40], dks[40];
     char str[512];
-    int error_code = MMC_ERROR;
 
     memset(hks, 0, sizeof(hks));
 
+    if (DEBUG_KEYS) {
+        DEBUG(DBG_MMC, "Host certificate   : %s\n", print_hex(str, host_cert,       92));
+        DEBUG(DBG_MMC, "Host nonce         : %s\n", print_hex(str, mmc->host_nonce, 20));
+    }
+
+    // send host cert + nonce
+    if (!_mmc_send_host_cert(mmc, agid, mmc->host_nonce, host_cert)) {
+
+        if ((mmc->drive_cert[1] & 0x01) && !(host_cert[1] & 0x01)) {
+            DEBUG(DBG_MMC | DBG_CRIT, "Certificate (id 0x%s) can not be used with bus encryption capable drive\n",
+                  print_hex(str, host_cert + 4, 6));
+        } else {
+            DEBUG(DBG_MMC | DBG_CRIT, "Host key / Certificate has been revoked by your drive ?\n");
+        }
+        return MMC_ERROR_CERT_REVOKED;
+    }
+
+    // receive mmc cert + nonce
+    if (!_mmc_read_drive_cert_challenge(mmc, agid, dn, mmc->drive_cert)) {
+        DEBUG(DBG_MMC | DBG_CRIT,
+              "Drive doesn't give its certificate\n");
+        return MMC_ERROR;
+    }
+
+    if (DEBUG_KEYS) {
+        DEBUG(DBG_MMC, "Drive certificate   : %s\n", print_hex(str, mmc->drive_cert, 92));
+        DEBUG(DBG_MMC, "Drive nonce         : %s\n", print_hex(str, dn, 20));
+    }
+
+    // verify drive certificate
+    if (!crypto_aacs_verify_drive_cert(mmc->drive_cert)) {
+        DEBUG(DBG_MMC | DBG_CRIT, "Drive certificate is invalid\n");
+        return MMC_ERROR;
+    }
+
+    // receive mmc key
+    if (!_mmc_read_drive_key(mmc, agid, dkp, dks)) {
+        DEBUG(DBG_MMC | DBG_CRIT, "Drive doesn't give its drive key\n");
+        return MMC_ERROR;
+    }
+
+    if (DEBUG_KEYS) {
+        DEBUG(DBG_MMC, "Drive key point     : %s\n", print_hex(str, dkp, 40));
+        DEBUG(DBG_MMC, "Drive key signature : %s\n", print_hex(str, dks, 40));
+    }
+
+    // verify drive signature
+    if (!_verify_signature(mmc->drive_cert, dks, mmc->host_nonce, dkp)) {
+        DEBUG(DBG_MMC | DBG_CRIT, "Drive signature is invalid\n");
+        return MMC_ERROR;
+    }
+
+    // sign
+    crypto_aacs_sign(host_cert, host_priv_key, hks, dn,
+                     mmc->host_key_point);
+
+    // verify own signature
+    if (!_verify_signature(host_cert, hks, dn, mmc->host_key_point)) {
+        DEBUG(DBG_MMC | DBG_CRIT, "Created signature is invalid ?\n");
+        return MMC_ERROR;
+    }
+
+    // send signed host key and point
+    if (!_mmc_send_host_key(mmc, agid, mmc->host_key_point, hks)) {
+        DEBUG(DBG_MMC | DBG_CRIT, "Error sending host signature\n");
+        DEBUG(DBG_MMC,  "Host key signature : %s\n", print_hex(str, hks, 40));
+        return MMC_ERROR;
+    }
+
+    // calculate bus key
+    if (bus_key) {
+        crypto_create_bus_key(mmc->host_key, dkp, bus_key);
+        if (DEBUG_KEYS) {
+            DEBUG(DBG_MMC, "Bus Key             : %s\n", print_hex(str, bus_key, 16));
+        }
+    }
+
+    return MMC_SUCCESS;
+}
+
+/*
+ *
+ */
+
+static int _read_vid(MMC *mmc, uint8_t agid, const uint8_t *bus_key, uint8_t *vid)
+{
+    uint8_t mac[16], calc_mac[16];
+    char str[512];
+
     DEBUG(DBG_MMC, "Reading VID from drive...\n");
 
+    if (_mmc_read_vid(mmc, agid, vid, mac)) {
+        if (DEBUG_KEYS) {
+            DEBUG(DBG_MMC, "VID                 : %s\n", print_hex(str, vid, 16));
+            DEBUG(DBG_MMC, "VID MAC             : %s\n", print_hex(str, mac, 16));
+        }
+
+        /* verify MAC */
+        crypto_aes_cmac_16(vid, bus_key, calc_mac);
+        if (memcmp(calc_mac, mac, 16)) {
+            DEBUG(DBG_MMC | DBG_CRIT, "VID MAC is incorrect. This means this Volume ID is not correct.\n");
+        }
+
+        return MMC_SUCCESS;
+    }
+
+    DEBUG(DBG_MMC | DBG_CRIT, "Unable to read VID from drive!\n");
+
+    return MMC_ERROR;
+}
+
+static int _read_pmsn(MMC *mmc, uint8_t agid, const uint8_t *bus_key, uint8_t *pmsn)
+{
+    uint8_t mac[16], calc_mac[16];
+    char str[512];
+
+    DEBUG(DBG_MMC, "Reading PMSN from drive...\n");
+
+    if (_mmc_read_pmsn(mmc, agid, pmsn, mac)) {
+        if (DEBUG_KEYS) {
+            DEBUG(DBG_MMC, "PMSN                : %s\n", print_hex(str, pmsn, 16));
+            DEBUG(DBG_MMC, "PMSN MAC            : %s\n", print_hex(str, mac, 16));
+        }
+
+        /* verify MAC */
+        crypto_aes_cmac_16(pmsn, bus_key, calc_mac);
+        if (memcmp(calc_mac, mac, 16)) {
+            DEBUG(DBG_MMC | DBG_CRIT, "PMSN MAC is incorrect. This means this Pre-recorded Medial Serial Number is not correct.\n");
+        }
+
+        return MMC_SUCCESS;
+    }
+
+    DEBUG(DBG_MMC | DBG_CRIT, "Unable to read PMSN from drive!\n");
+
+    return MMC_ERROR;
+}
+
+static int _read_data_keys(MMC *mmc, uint8_t agid, const uint8_t *bus_key,
+                           uint8_t *read_data_key, uint8_t *write_data_key)
+{
+    uint8_t encrypted_read_data_key[16], encrypted_write_data_key[16];
+    char str[512];
+
+    DEBUG(DBG_MMC, "Reading data keys from drive...\n");
+
+    if (_mmc_read_data_keys(mmc, agid, encrypted_read_data_key, encrypted_write_data_key)) {
+        if (read_data_key) {
+            crypto_aes128d(bus_key, encrypted_read_data_key, read_data_key);
+            if (DEBUG_KEYS) {
+                DEBUG(DBG_MMC, "READ DATA KEY       : %s\n", print_hex(str, read_data_key, 16));
+            }
+        }
+        if (write_data_key) {
+            crypto_aes128d(bus_key, encrypted_write_data_key, write_data_key);
+            if (DEBUG_KEYS) {
+                DEBUG(DBG_MMC, "WRITE DATA KEY      : %s\n", print_hex(str, write_data_key, 16));
+            }
+        }
+
+        return MMC_SUCCESS;
+    }
+
+    DEBUG(DBG_MMC | DBG_CRIT, "Unable to read data keys from drive!\n");
+
+    return MMC_ERROR;
+}
+
+int mmc_read_auth(MMC *mmc, const uint8_t *host_priv_key, const uint8_t *host_cert,
+                  int request, uint8_t *p1, uint8_t *p2)
+{
+    uint8_t agid = 0, bus_key[16];
+    int error_code;
+
     _mmc_invalidate_agids(mmc);
 
     if (!_mmc_report_agid(mmc, &agid)) {
@@ -1022,101 +1216,81 @@ int mmc_read_vid(MMC *mmc, const uint8_t *host_priv_key, const uint8_t *host_cer
     }
     DEBUG(DBG_MMC, "Got AGID from drive: %d\n", agid);
 
-    if (!PATCHED_DRIVE) do {
-
-        if (DEBUG_KEYS) {
-            DEBUG(DBG_MMC, "Host certificate   : %s\n", print_hex(str, host_cert,       92));
-            DEBUG(DBG_MMC, "Host nonce         : %s\n", print_hex(str, mmc->host_nonce, 20));
-        }
-
-        // send host cert + nonce
-        if (!_mmc_send_host_cert(mmc, agid, mmc->host_nonce, host_cert)) {
-            DEBUG(DBG_MMC | DBG_CRIT, "Host key / Certificate has been revoked by your drive ?\n");
-            error_code = MMC_ERROR_CERT_REVOKED;
-            break;
-        }
-
-        // receive mmc cert + nonce
-        if (!_mmc_read_drive_cert(mmc, agid, dn, dc)) {
-            DEBUG(DBG_MMC | DBG_CRIT,
-                  "Drive doesn't give its certificate\n");
-            break;
-        }
-
-        if (DEBUG_KEYS) {
-            DEBUG(DBG_MMC, "Drive certificate   : %s\n", print_hex(str, dc, 92));
-            DEBUG(DBG_MMC, "Drive nonce         : %s\n", print_hex(str, dn, 20));
-        }
-
-        // verify drive certificate
-        if (!crypto_aacs_verify_drive_cert(dc)) {
-            DEBUG(DBG_MMC | DBG_CRIT, "Drive certificate is invalid\n");
-            break;
-        }
-
-        // receive mmc key
-        if (!_mmc_read_drive_key(mmc, agid, dkp, dks)) {
-            DEBUG(DBG_MMC | DBG_CRIT, "Drive doesn't give its drive key\n");
-            break;
-        }
-
-        if (DEBUG_KEYS) {
-            DEBUG(DBG_MMC, "Drive key point     : %s\n", print_hex(str, dkp, 40));
-            DEBUG(DBG_MMC, "Drive key signature : %s\n", print_hex(str, dks, 40));
-        }
-
-        // verify drive signature
-        if (!_verify_signature(dc, dks, mmc->host_nonce, dkp)) {
-            DEBUG(DBG_MMC | DBG_CRIT, "Drive signature is invalid\n");
-            break;
-        }
-
-        // sign
-        crypto_aacs_sign(host_cert, host_priv_key, hks, dn,
-                         mmc->host_key_point);
-
-        // verify own signature
-        if (!_verify_signature(host_cert, hks, dn, mmc->host_key_point)) {
-            DEBUG(DBG_MMC | DBG_CRIT, "Created signature is invalid ?\n");
-            break;
-        }
-
-        // send signed host key and point
-        if (!_mmc_send_host_key(mmc, agid, mmc->host_key_point, hks)) {
-            DEBUG(DBG_MMC | DBG_CRIT, "Error sending host signature\n");
-            DEBUG(DBG_MMC,  "Host key signature : %s\n", print_hex(str, hks, 40));
-            break;
-        }
-
-    } while (0);
-
-    if (_mmc_read_vid(mmc, agid, vid, mac)) {
-        DEBUG(DBG_MMC, "VID: %s\n", print_hex(str, vid, 16));
-        DEBUG(DBG_MMC, "MAC: %s\n", print_hex(str, mac, 16));
-        /* TODO: verify MAC */
-
-        /* read pmsn */
-        if (_mmc_read_pmsn(mmc, agid, pmsn, mac)) {
-            DEBUG(DBG_MMC, "PMSN: %s\n", print_hex(str, pmsn, 16));
-            DEBUG(DBG_MMC, "MAC:  %s\n", print_hex(str, mac,  16));
-            /* TODO: verify MAC */
-        } else {
-            memset(pmsn, 0, 16);
-            DEBUG(DBG_MMC | DBG_CRIT, "Unable to read PMSN from drive!\n");
-        }
+    /*
+     * NOTE: It seems that at least some drives require a new AACS-Auth every time
+     * keys are being read
+     */
 
+    error_code = _mmc_aacs_auth(mmc, agid, host_priv_key, host_cert, bus_key);
+    if (error_code) {
         _mmc_invalidate_agid(mmc, agid);
-
-        return MMC_SUCCESS;
+        return error_code;
     }
 
-    DEBUG(DBG_MMC | DBG_CRIT, "Unable to read VID from drive!\n");
+    switch (request) {
+        case MMC_READ_VID:
+            error_code = _read_vid(mmc, agid, bus_key, p1);
+            break;
+        case MMC_READ_PMSN:
+            error_code = _read_pmsn(mmc, agid, bus_key, p1);
+            break;
+        case MMC_READ_DATA_KEYS:
+            error_code = _read_data_keys(mmc, agid, bus_key, p1, p2);
+            break;
+        default:
+            DEBUG(DBG_MMC | DBG_CRIT, "unknown mmc_read_auth() request %d\n", request);
+            error_code = MMC_ERROR;
+            break;
+    }
 
     _mmc_invalidate_agid(mmc, agid);
 
     return error_code;
 }
 
+/*
+ *
+ */
+
+int mmc_read_drive_cert(MMC *mmc, uint8_t *drive_cert)
+{
+    uint8_t buf[116];
+
+    if (mmc->drive_cert[0] == 0x01) {
+        memcpy(drive_cert, mmc->drive_cert, 92);
+        return MMC_SUCCESS;
+    }
+
+    if (!mmc->read_drive_cert) {
+        DEBUG(DBG_MMC | DBG_CRIT, "Drive does not support reading drive certificate\n");
+    }
+
+    if (!_mmc_report_key(mmc, 0, 0, 0, 0x38, buf, 116)) {
+        if (mmc->read_drive_cert) {
+            DEBUG(DBG_MMC | DBG_CRIT, "Failed reading drive certificate\n");
+        }
+        return MMC_ERROR;
+    }
+
+    if (!crypto_aacs_verify_drive_cert(buf + 4)) {
+        DEBUG(DBG_MMC | DBG_CRIT, "Drive certificate is invalid\n");
+        return MMC_ERROR;
+    }
+
+    memcpy(mmc->drive_cert, buf + 4, 92);
+    memcpy(drive_cert, buf + 4, 92);
+
+    return MMC_SUCCESS;
+}
+
+const uint8_t *mmc_get_drive_cert(MMC *mmc)
+{
+    if (mmc->drive_cert[0] == 0x01) {
+        return mmc->drive_cert;
+    }
+    return NULL;
+}
+
 uint8_t *mmc_read_mkb(MMC *mmc, int address, int *size)
 {
     uint8_t agid = 0;
